/*****************************************************************
//
//      csparse.h
//
//      This file contains a simple interface to some of the
//      routines in the SPARSKIT library.  
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 28, 1994
//
******************************************************************/

/*===========================================================================
			FUNCTIONS :
			----------

rperm_matrix :
transpose_matrix : 	calculates the transp of the matrix
multiply matrices : 	multiplies 2 sparse matrices
add_matrices : 		adds 2 sparse matrices
substract_matrices : 	substracts 2 sparse matrices
write_matrices : 	writes a sparse matrix in a Harwell-Boeing file
read_matrix : 		reads a matrix stored in a Harwell-Boeing file
submat_matrix :		extracts a submatrix from a matrix
filter	:		filters the elements from a matrix according to 
			their magnitude
csort  : 		sorts the elements in increasing order of columns
getdia : 		extracts a specified diagonal from a matrix.
getu :			extracts the upper triangular part of a matrix
getl :			extracts the lower triangular part of a matrix
getbwd :	   	gives the bandwith of the upper and the lower part 
			of the mat
cnrms : 		computes the norms of the columns
roscal : 		scales the rows of a matrix by their norms
coscal : 		scales the columns of a matrix by their norms
getbwdu :	   	gives the bandwith of the upper part of the matrix
getbwdl :	   	gives the bandwith of the lower part of the matrix
gen57pt_matrix :	generates a 5-point and 7-point matrix
get_dim : 		gives the dimensions of a sparse matrix stored in a
			Harwell-Boeing file
indset0 :		greedy algorithm for independent set ordering
indset1 :		idem  but using minimal degree traversal
multic,colors,colors2 : greedy algorithm for multicoloring (the only difference
			between these functions is what they return)

FUNCTIONS ONLY USED BY SOLVE_SYSTEM (not available in spmath)

amux_matrix :		multilplies a matrix with a vector
atmux_matrix :		multiplies the transpose of a matrix with a vector
ilut_prec, iluk_prec : 	preconditionner
ilud_prec, ilu0_prec :  preconditionner
lusol_prec, lutsol_prec preconditionner
msrcsr_format :		create a matrix in compressed row format from a matrix
			in modified row format 
csrmsr_format :		create a matrix in modified row format from a matrix 
			in compressed row format
run*_solver :		these are 10 differents functions to run the solvers.

===========================================================================*/



#include <stdio.h>
#include <malloc.h>
#include <assert.h>

void readmt_ ();
void gen57pt_();
void submat_();
void filter_();
void csort_();
void getdia_();
void getu_();
void getl_();
void getbwd_();
void blkfnd_();
void cnrms_();
void coscal_();
void roscal_();
void addblk_();
void get1up_();
void infdia_();
void prtmt_ ();
void amub_ ();
void aplb_ ();
void aplsb_ ();
void transp_ ();
void get_dim ();
void rperm_ ();
void cperm_();
void dperm_();
void amux_();
void atmux_();

void ilut_();
void ilutp_();
void iluk_();
void ilu0_();
void ilud_();
void msrcsr_();
void csrmsr_();
void lusol_();
void lutsol_();
void cg_();
void bcg_();
void dbcg_();
void bcgstab_();
void fom_();
void cgnr_();
void tfqmr_();
void gmres_();
void fgmres_();
void dqgmres_();

void runrc_();
void multic_();
void indset0_();
void indset1_();

/*
  Note: Strangely, fortran subroutines with underscores
 already in the name are not given an additional underscore
 at the end.
*/

void bandwidth_();
void nonz_();
void diag_domi_();
void frobnorm_();
void ansym_();
void distaij_();
void skyline_();
void distdiag_();
void bandpart_();
void n_imp_diag_();
void nonz_lud_();
void avnz_col_();

void csrcsc_();

void read_matrix();
void submat_matrix();
void filter_matrix();
void csort_matrix();
void getdia_matrix();
void getu_matrix();
void getl_matrix();
void blkfnd_matrix();
void cnrms_matrix();
void coscal_matrix();
void roscal_matrix();
void addblk_matrices();
void get1up_matrix();
void infdia_matrix();
void gen57pt_matrix();
void write_matrix ();
void multiply_matrices ();
void add_matrices ();
void subtract_matrices ();
void transpose_matrix ();
void rperm_matrix ();
void cperm_matrix();
void dperm_matrix();
void amux_matrix();
void atmux_matrix();
void MAIN_ ();

void ilut_prec();
void ilud_prec();
void iluk_prec();
void ilu0_prec();
void msrcrs_format();
void csrmsr_format();
void lusol_prec();
void lutsol_prec();

void rungmres_solver();
void runfgmres_solver();
void rundqgmres_solver();
void runfom_solver();
void runtfqmr_solver();
void runbcgstab_solver();
void runcgnr_solver();
void rundbcg_solver();
void runbcg_solver();
void runcg_solver();

void multic_matrix();
void indset0_matrix();
void indeset1_matrix();

void bandwidth();
void nonz();
void diag_dominance();
void frobnorm();
void ansym();
void distaij();
void skyline();
void distdiag();
void bandpart();
void n_important_diag();
void nonzero_lud();
void avnz_column();
void csrcsc();
