//****************************************************************
//
//    Builtin Functions
//
//    Description:
//	Contains all of the builtin functions.
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 18, 1995
//
//****************************************************************

/*===========================================================================
				FUNCTIONS :
				----------

function(arg,f) : 		applies the function f to arg
rperm(mat,perm) : 		permutes mat with perm
eye(arg) : 			returns an identity matrix
zeroes(arg) : 			returns a matrix composed with zeroes
set_element(name,args,elt) : 	sets the element of the matrix name whose
				coordinates are stored in args to the value elt
get_element(name,args) : 	gets the element of the matrix name whose
				coordinates are stored in args
help(topic) : 			provides a documentation on the topic
readmt(file) : 			reads the matrix stored in the Boeing-Harwell
				file
prtmt(mat,file) : 		prints the matrix mat int he Boeing-Harwell file
submat :			extracts a submatrix from a matrix
filter	:			filters the elements from a matrix according to 
				their magnitude
csort :				sorts the elements in increasing order 
				of columns
getdia : 			extracts a specified diagonal from a matrix.
rmdia : 			removes a specified diagonal from a matrix.
getu :				extracts the upper triangular part of a matrix
getl :				extracts the lower triangular part of a matrix
getbwd :	   		gives the bandwith of the upper and the lower
				part of the mat
getbwdu :	   		gives the bandwith of the upper part of matrix
getbwdl :	   		gives the bandwith of the lower part of matrix
cnrms : 			computes the norms of the columns
roscal : 			scales the rows of a matrix by their norms
coscal : 			scales the columns of a matrix by their norms
gen57pt :			generates a 5-point or a 7-point matrix
vec : 				creates a vector 
solve_system :			Solve a linear system Ax=y where A is sparse.
indset0 :			greedy algorithm for independent set ordering
indset1 :			idem  but using minimal degree traversal
multic,colors,colors2 : 	greedy algorithm for multicoloring (the only 
				difference between these functions is what 
				they return)

===========================================================================*/

#ifndef FUNCTIONS_H
#define FUNCTIONS_H

#include "string.h"
#include <iostream.h>
#include <fstream.h>
#include <stdlib.h>
#include "value.h"
#include "list.h"
#include "symtab.h"

extern "C" { typedef double (*ffunc)(double); }

extern symtab *syms;

value * function (value *, ffunc, int &);
value * readmt (value *, int &);
value * gen57pt(value *, value *, value *, value *, int&);
value * submat(value *, value *, value *, value *, value *, int&);
value * filter(value *, value *, value *, value *, int& );
value * csort (value *, value *, int&);
value * rmdia(value *, value *, int&);
value * getdia (value *, value *, int&);
value * getu (value *, int&);
value * getl (value *, int&);
value * getbwd (value *, int&);
value * getbwdu (value *, int&);
value * getbwdl (value *, int&);
value * blkfnd (value *, int &);
value * cnrms (value *, value *, int&);
value * coscal (value *, value *, int &);
value * roscal (value *, value *, int &);
value * addblk (value *, value *, value *, value *, int&);
value * get1up (value *, int&);
value * prtmt (value *, value *, int &);
value * rperm (value *, value *, int &);
value * cperm (value *, value *, int &);
value * dperm (value *, value *, value *, int &);
value * vec (value *, value *, value *, int &);
value * eye (value *, int &);
value * zeroes (value *, int &);
value * zeroes (value *, value *, int &);
value * spzeroes (value *, int &);
value * spzeroes (value *, value *, int &);
value * ones (value *, int &);
value * ones (value *, value *, int &);
value * get_element (char *, list *, int &);
value * set_element (char *, list *, value *, int &);
value * get_range(char *, value *, value *, value *, int &);
value * help (char *);
value * spit_file (char *);

value * solve_system (value *, value *, value *, value *, value *, value *, value *, value *, value *, value *, int&);

// The 3-arg version of solve system (using the stored variables)
value * solve_system (value *, value *, value *, int&);

value * rando (value *, value *, value *, value *, int&);
value * rando (value *, value *, value *, int &);
value * init(value *, value *, value *, int &);
value * viewhb(value *, int &);
value * drawmat (value *, int &);

value * multic (value *, int&);
value * colors (value *, int&);
value * colors2 (value *, int&, int&);
value * il_multic (value *, int&);
value * iord_multic (value *,int &);
value * indset0(value *, int&);
value * indset1(value *, int&);

#endif














