//****************************************************************
//
//      Class history
//
//      List is a class for implementing a history list. 
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 12, 1994
//
//****************************************************************

/*==========================================================================
			PRIVATE :
			--------

    int size;               // Size of the history list
    int current;            // Number of the most recent item.
    string * hist;          // Pointer to the list itself.

Each entry of the list contains :
	- the number of the line
	- the line

			FUNCTIONS :
			----------
constructors
destructor

get_item(i): 	gets the ith iten of the list : the number and the line
put_item(it) : 	puts the item it at the end of the history list
out : 		outputs the history list

==========================================================================*/


#include "spmath.h"

//************************************************************************
//    Member constructors
//************************************************************************

history::history (int n) {
  // Create an empty history list with enough space for
  // n entries.

  size = n;
  current = 0;
  hist = new string [n+1];

} // end history


//************************************************************************
//    Member Selectors
//************************************************************************

/*=========================================================================
#			GET_ITEM :
#			---------
# Entries : 	an integer : num
#
# Returns : 	an item of the history list
#
# Details : 	- if the number is negative, then returns the (num)'th item 
#		  from the end
#		- checks if such a number exists in the history list
#		- if the number is positive, returns the (num)'th item 
#		  from the beginning
#
=========================================================================*/

string history::get_item (int num) {
  // Get an item from the history list (the number and the line which 
  // corresponds)

  // If the num is neg, then count backwards from the end.
  if (num < 0)
    num = current + num + 1;

  if (num > current  ||  num <= current - size || num <= 0) {
    sp_out("Sorry, no event number "); sp_out(num); sp_out("\n");
    cout << "Sorry, no event number " << num << endl;
    return "";
  }

  return hist[num - ((num-1)/size)*size];

} // end get_item.

//************************************************************************
//    Member Functions
//************************************************************************

/*=========================================================================
#				PUT_ITEM :
#				---------
# Entries : 	a string : item (the item to put into the history list)
#
# Returns : 	nothing
#
# Details :     It puts an item at the end of the history list and updates 
#		the current pointer
#
=========================================================================*/

void history::put_item (string & item) {
  // Put an item into the history list.

  current += 1;
  hist[current - ((current-1)/size)*size] = item;

} // end put

/*=========================================================================
#				OUT :
#				----
# Entries : 	nothing
#
# Returns : 	nothing
#
# Details : 	It outputs the history list.
#		For each item, it prints its number and its contents
#
=========================================================================*/

void history::out () {
  // Output the history list.

  int i;

  if (current <= size) {
    for (i = 1; i <= current; i++) {
      sp_out(i); sp_out("\t");
      cout << i << "\t";

      output(hist[i]); 
      sp_out("\n");
      cout << endl;
    } 
    return;
  } 

  int end = current - ((current-1)/size)*size;

  for (i = end+1; i <= size; i++) {
    int cur = (i-end) - size + current;

    sp_out(cur); sp_out("\t");
    cout << cur << "\t";

    output (hist[i]);

    sp_out("\n");
    cout << endl;
  }

  for (i = 1; i <= end; i++) {
    int cur = i + ((current-1)/size)*size;

    sp_out(cur); sp_out("\t");
    cout << cur << "\t";

    output (hist[i]);

    sp_out("\n");
    cout << endl;
  }

} // end out

//************************************************************************
//    Functions
//************************************************************************

void output (history & hist) {
  // Output the history list hist.

  hist.out();
} // end output
