// ****************************************************************
//
//      Class link
// 
//      Link is a class that will hold one element and one 
//      pointer of a linked list.  For the most part, the
//      the only direct user of this class should be the
//      class list. 
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 10, 1995
//
// ****************************************************************

/*=========================================================================
			PRIVATE :
			--------
    entry thing;	
    link *pointer;	// pointer to the linked list

			FUNCTIONS :
			---------
constructors
destructor

get_item : 		returns the item field 
get_pointer : 		returns the pointer field (ptr to the linked list)
insert(en) : 		inserts a new link at the beginning 
set_pointer(p) : 	sets the pointer field to p

=========================================================================*/

#include "link.h"


//************************************************************
//  Member Constructors
//************************************************************


link::link (entry x, link* y) {
  // Creates a link

  thing = x;
  pointer = y;
} // end link

link::link () {
  // Create an empty link. But do  not initialize it

  pointer = NULL;
} // end link

link::link (entry x) {
  // Create a link.
  thing = x;
  pointer = NULL;
} // end link

link::link (const link &x) {
  // Copy constructor

  thing = x.get_item();
  pointer = x.get_ptr();

} // end link

//************************************************************
//  Member Selectors.
//************************************************************

/*=========================================================================
#			GET_ITEM :
#			---------
# Entries : 	nothing
#
# Returns : 	the item field of the link
#
=========================================================================*/

entry link::get_item () const {
  // returns the item field of a link.
  return thing;
} // end get_item

/*=========================================================================
#			GET_PTR :
#			--------
# Entries : 	nothing
#
# Returns : 	the pointer field of the link
#
=========================================================================*/

link* link::get_ptr () const {
  // returns the pointer field of a link.

  return  pointer;
} // end get_ptr

  
//************************************************************
//  Member Mutators
//************************************************************

/*=========================================================================
#				INSERT :
#				-------
# Entries : 	an entry : val (the entry to insert)
#
# Returns : 	the new link
#
# Details :	old link :					New one 
#				------				-------
#		pointer	------>	| E1 |		pointer ------>	|  E1 |
#				------				-------
#		   		| E2 |				| val |
#				------				-------
#				| ...|				|  E2 |
#				------				-------
#				| En |				| ... |
#				------				-------
#								|  En |
#								-------	
#
=========================================================================*/

link* link::insert (entry val) {
  // insert a new link

  pointer = new link (val,pointer);
  
  return pointer;
} // end insert

/*=========================================================================
#				SET_POINTER :
#				-----------
# Entries : 	a link
#
# Returns : 	nothing
#
# Details :	it sets the pointer to point
#
=========================================================================*/

void link::set_pointer (link *point) {
  // set the pointer to point.

  pointer = point;
} // end set_pointer
