//****************************************************************
//
//      Class list
//
//      List is a class for implementing linked lists.  It is
//      written as a template for supporting multiple types of
//      data.  
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 12, 1995
//
//****************************************************************

/*============================================================================
			PRIVATE :
			--------
link * current_pointer
link * previous_pointer
link * top_pointer

			FUNCTIONS :
			----------
constructors
destructor

get_ptr : 	returns the pointer to the first element of the list
top : 		returns the first element of the list
pop : 		returns the first element of the list and remove it from the
		list
push(v) : 	puts v onto the front of the list
empty : 	returns 1 if the list is empty
end : 		returns 1 if the current pointer is at the last element of the
		list
init_curr_prev :initializes the current pointer to the top item of the
		list and initialize the previous pointer to NULL
get_current : 	returns the value of the item that is in the link pointed by the
		current pointer
get_next : 	returns the value of the item that is in the link following the
		link pointed by the current pointer
get_previous : 	returns the value of the item that is in the link pointed by the
		previous pointer
remove_top : 	removes the first item of the list
remove_current :removes the item which is in the link pointed by the current
		pointer
remove_all : 	removes all the items of the list

============================================================================*/

#ifndef LIST_H
#define LIST_H

#include <assert.h>
#include "entry.h"
#include "link.h"


class list {

  public:
    // constructors
    list (link*);
    list (const list &);
    list ();

    // selectors
    link* get_ptr () const;

    // member functions
    void push (entry);
    entry pop ();
    entry top ();
    int empty ();
    int end ();
    void init_curr_prev ();
    entry get_current ();
    entry get_next ();
    entry get_previous ();
    void remove_top();
    void remove_current();
    void remove_all();

  private:

    link *top_pointer;
    link *current_pointer;
    link *previous_pointer;
}; // end class list


#endif
