//****************************************************************
//
//    Class symtab
//
//    Description:
//	Symtab is a class for implementing a symbol table.  
//	The symbol table is implemented as an array of linked 
//      lists of entries. (see class entry.)  Each linked list
//	corresponds to a unique hash number. 
//
//    Remark: spmath uses this class (symtab) to store the variables and the
//	      functions
//
//      If you discover any bugs, it would be very helpful if
//      you sent me e-mail.  Thank you.
//
//      Author:                Peter Braaten
//      e-mail:                braa0007@mermaid.micro.umn.edu
//      Date last modified:    July 18, 1995
//
//****************************************************************

/*===========================================================================
			PRIVATE :
			---------
    list syms[SIZE];	// array of entry lists 
    symtab *parent;	// each symtab has a parent which is also a symtab

			FUNCTIONS :
			----------

constructors
destructor

is_root : 	returns 1 if the parent is NULL
get_parent :	returns the pointer to the parent symbol table
contains(n) : 	returns 1 if the symbol table contains an entry whose name is n
lookup(n) : 	verifies if the symbol table contains an entry whose name is n
		and returns this entry
out() : 	outputs the contents of the symtab
insert(e) : 	inserts the entry e into the symbol table
remove(n) : 	removes the entry with the name n
remove_all : 	removes all the entries of the symbol table
hash(n) : 	assigns a number to each name (this will be the number of the
		list in which an entry with this name will be stored)

===========================================================================*/




#ifndef SYMTAB_H
#define SYMTAB_H
#define SIZE 207

#include <assert.h>
#include <string.h>
#include "entry.h"
#include "list.h"

class symtab {

  public:
    // Constructors
    symtab();
    symtab(symtab *);

    // Selectors
    entry lookup (const char *);
    int contains (const char *);
    void out ();
    void out_fancy ();
    void out_alias ();
    int is_root();
    symtab* get_parent();

    // Mutators
    void insert (entry);
    void remove (const char *);
    void remove_all ();

  private:
    // Data
    list syms[SIZE];	// array of entry lists 
    symtab *parent;	// each symtab has a parent which is also a symtab
}; // end class symtab

unsigned int hash (const char *);


#endif
